#!/usr/bin/env python3
import argparse
import base64
import copy
import json
import os
import shutil
import sys
import time
from cmd import kubectl_command
from cmd import set_kubectl
from configparser import ConfigParser
from itertools import chain
from pathlib import Path

import yaml

from api_request import assert_request_success
from api_request import local_node_request
from api_request import remote_node_request_with_retry
from api_request import set_cert_manager_ns
from constants import COMPATIBILITY_FILE
from constants import Certificate
from constants import DELETE_CERT
from constants import DELETE_CSR
from constants import DELETE_PI
from constants import GET_ALL_CERTS
from constants import GET_CLOUDNATIVE_PLATFORM_CERTS
from constants import GET_FEATURES_INTELLIGENCE_CONFIG
from constants import GET_PI_CERTS
from constants import GET_PLATFORM_HEALTH
from constants import NAPP_CERT_EGRESS_ENTITIES
from constants import NAPP_CERT_NONEGRESS_ENTITIES
from constants import PATH_CA_CERTIFICATE_DIR
from constants import POST_APPLY_KAFKA_CLIENT_CERT
from constants import POST_CLOUDNATIVE_PLATFROM_CERT
from constants import POST_GENERATE_CSR
from constants import POST_REGISTER_PI_CERT
from constants import POST_SELF_SIGN_CSR
from constants import PRTON_PORT
from constants import RELEASE_USED_CERTIFICATE
from constants import WORK_DIR
from constants import YAML_DEPLOYMENT_PATCH
from constants import YAML_SECRET_HARBOR
from constants import YAML_VOLUME_DEFINITION
from constants import YAML_VOLUME_MOUNT
from logger import get_logger
from logger import setup_logger

lg = get_logger(__name__)


def _get_min_max_k8s_supported_version():
    chart_file_dir = Path(__file__).parent.resolve().parent
    compatibility_file_path = "%s/%s" % (chart_file_dir, COMPATIBILITY_FILE)
    parser = ConfigParser()
    with open(compatibility_file_path) as lines:
        lines = chain(("[DEFUALT]",), lines)  # This line does the trick.
        parser.read_file(lines)
    return parser.get('DEFUALT', 'k8s-min-version'), parser.get('DEFUALT', 'k8s-max-version')


def validate_k8s_compatibility():
    command = "version -ojson"
    rc, out, error = kubectl_command(command)
    if rc != 0:
        lg.error("Failed to validate kubernetes version %s" % error)
        raise RuntimeError("Failed to validate kubernetes version %s" % error)
    json_str = json.loads(out)
    server_version_dict = json_str['serverVersion']
    k8s_version = "%s.%s" % (server_version_dict["major"], server_version_dict["minor"])
    k8s_min_version, k8s_max_version = _get_min_max_k8s_supported_version()
    lg.info("Deployed k8 version %s, minimum supported version %s, max supported version %s" %
            (k8s_version, k8s_min_version, k8s_max_version))
    if k8s_min_version <= k8s_version <= k8s_max_version:
        lg.info("Deployed k8s version %s is supported" % k8s_version)
        return
    lg.error("Deployed k8s version %s is not supported" % k8s_version)
    raise RuntimeError("Deployed k8s version %s is not supported" % k8s_version)


def _get_ingress_fqdn(namespace: str) -> str:
    command = r"get httpproxy cloudnative-platform-ingress -n %s " \
              r"-ojsonpath='{.spec.virtualhost.fqdn}' " % namespace
    rc, out, error = kubectl_command(command)
    if rc != 0:
        lg.error("Failed to get ingress FQDN with error msg: %s" % error)
        raise RuntimeError("Failed to get FQDN with error msg: %s" % error)
    lg.info("Ingress FQDN: %s" % out)
    return out.replace("'", "")


def _wait_util_platform_ready(host: str, max_retry: int = 25,
                              interval: float = 20):
    attempt = 0
    while attempt < max_retry:
        ret, _ = remote_node_request_with_retry(host, "GET",
                                                GET_PLATFORM_HEALTH,
                                                max_retry=1)
        if ret is not None and "health" in ret and ret['health'] == "UP":
            lg.info("All Platform services are running")
            return
        attempt = attempt + 1
        time.sleep(interval)

    lg.error("Platform services are unhealthy")
    raise RuntimeError("Platform services are unhealthy")


# this is needed for H -> I migration
def _get_features_intelligence_config(host: str):
    lg.info("Get Feature Intelligence Config")
    assert_request_success(
        remote_node_request_with_retry(host, "GET",
                                       GET_FEATURES_INTELLIGENCE_CONFIG,
                                       max_retry=5),
        "GET: %s" % GET_FEATURES_INTELLIGENCE_CONFIG)


# Create a private key and self-signed certificate for common agent/pace agent.
# It's used as keystore
def _register_kafka_client_cert():
    body = {
        "display_name": Certificate.display_name,
        "algorithm": Certificate.algorithm,
        "key_size": Certificate.key_size,
        "subject": Certificate.subject
    }
    header = {'X-NSX-Username': "admin"}
    # 1. generate a csr
    ret = assert_request_success(local_node_request(PRTON_PORT, "POST",
                                                    POST_GENERATE_CSR, body,
                                                    header),
                                 "POST: %s" % POST_GENERATE_CSR)
    csr_id = ret['id']
    # 2. self signed the csr
    ret = assert_request_success(
        local_node_request(PRTON_PORT, "POST",
                           POST_SELF_SIGN_CSR.replace("<id>", csr_id), None,
                           header),
        "POST: %s" % POST_SELF_SIGN_CSR)
    cert = ret['id']
    # 3. apply the cert
    assert_request_success(
        local_node_request(PRTON_PORT, "POST",
                           POST_APPLY_KAFKA_CLIENT_CERT.replace("<id>", cert),
                           None, header),
        "POST: %s" % POST_APPLY_KAFKA_CLIENT_CERT)
    # 4. delete the csr
    assert_request_success(
        local_node_request(PRTON_PORT, "DELETE",
                           DELETE_CSR.replace("<id>", csr_id), None, header),
        "DELETE: %s" % DELETE_CSR)
    lg.info("kafka client cert is generated and imported")

    # sleep for a while to mitigate the time sync issue between manager and k8s
    time.sleep(30)


# release K8S_MSG_CLIENT certificate on manager
def _unregister_kafka_client_cert():
    # in case the k8s_msg_client has been attached by other role's certificate
    header = {'X-NSX-Username': "admin", 'X-Allow-Overwrite': True}
    ret = assert_request_success(
        local_node_request(PRTON_PORT, "GET", GET_ALL_CERTS, None, header),
        "GET: %s" % GET_ALL_CERTS)
    # release K8S_MSG_CLIENT certificate on manager
    for cert in ret['results']:
        used_by = cert['used_by']
        if used_by:
            for ele in used_by:
                if Certificate.service_type in ele['service_types']:
                    body = {
                        "node_id": ele['node_id'],
                        "service_type": Certificate.service_type
                    }
                    cert_id = cert['id']
                    # release certificate
                    assert_request_success(
                        local_node_request(PRTON_PORT, "POST",
                                           RELEASE_USED_CERTIFICATE.
                                           replace("<id>", cert_id),
                                           body=body),
                        "POST: %s" % RELEASE_USED_CERTIFICATE)

    # delete system-generated certificate used as K8S_MSG_CLIENT
    for cert in ret['results']:
        if cert['_create_user'] == 'admin' and \
                cert['resource_type'] == "certificate_self_signed" and \
                cert['display_name'] == Certificate.display_name:
            cert_id = cert['id']
            # delete certificate
            assert_request_success(
                local_node_request(PRTON_PORT, "DELETE",
                                   DELETE_CERT.
                                   replace("<id>", cert_id), None, header),
                "DELETE: %s" % DELETE_CERT)


# Import NAPP certificates into NSX Manager.
# And create the related PIs
def _register_cloudnative_platform_certs(host: str):
    headers = {'X-NSX-Username': "admin"}
    ret = assert_request_success(
        remote_node_request_with_retry(host, "GET",
                                       GET_CLOUDNATIVE_PLATFORM_CERTS, None,
                                       headers),
        "GET: %s" % GET_CLOUDNATIVE_PLATFORM_CERTS)
    for cert in ret['results']:
        body = {}
        pi = cert['alias']
        pem = cert['pem_encoded']
        if cert['used_by'] in NAPP_CERT_EGRESS_ENTITIES:
            body = {
                "name": pi,
                "node_id": "nsx.application.platform",
                "role": "napp_role",
                "is_protected": "true",
                "certificate_pem": pem
            }
        elif cert['used_by'] in NAPP_CERT_NONEGRESS_ENTITIES:
            # the least privileges
            body = {
                "name": pi,
                "node_id": "nsx.application.platform",
                "role": "network_engineer",
                "is_protected": "true",
                "certificate_pem": pem
            }
        if body:
            lg.info("PI: %s", pi)
            lg.info("Cert: %s", cert)
            assert_request_success(
                local_node_request(PRTON_PORT, "POST",
                                   POST_REGISTER_PI_CERT, body, headers),
                "POST: %s" % POST_REGISTER_PI_CERT)


# Delete NAPP PIs and related certificates from NSX Manager
def _unregister_cloudnative_platform_certs():
    # delete k8s side certificates on manager
    ret = assert_request_success(
        local_node_request(PRTON_PORT, "GET", GET_PI_CERTS),
        "GET: %s" % GET_PI_CERTS)
    # in case the PI is modified by other role, add X-Allow-Overwrite
    headers = {'X-NSX-Username': "admin", 'X-Allow-Overwrite': True}
    for pi in ret['results']:
        if pi['node_id'] == "nsx.application.platform":
            assert_request_success(
                local_node_request(PRTON_PORT, "DELETE",
                                   DELETE_PI.replace("<id>", pi['id']), None,
                                   headers), "DELETE %s" % DELETE_PI)
            assert_request_success(
                local_node_request(PRTON_PORT, "DELETE",
                                   DELETE_CERT.replace("<id>",
                                                       pi['certificate_id']),
                                   None, headers), "DELETE: %s" % DELETE_CERT)


# Push NSX Manager certificates into NAPP
def _register_manager_certs(host: str):
    node_certs = []
    kafka_certs = []
    ret = assert_request_success(
        local_node_request(PRTON_PORT, "GET", GET_ALL_CERTS),
        "GET: %s" % GET_ALL_CERTS)
    for cert in ret['results']:
        used_by = cert['used_by']
        if used_by:
            for ele in used_by:
                if "API" in ele['service_types'] and cert['resource_type'] != "certificate_ca":
                    lg.info(ele)
                    node_certs.append(cert)
                    break
                elif "K8S_MSG_CLIENT" in ele['service_types']:
                    lg.info(ele)
                    kafka_certs.append(cert)
                    break

    lg.info("Found %s node certs on manager cluster" % str(len(node_certs)))
    lg.info("Found %s kafka client certs on manager cluster" %
            str(len(kafka_certs)))

    # 1. push manager node certs to k8s platform
    if node_certs:
        _push_certs(host, node_certs, "NSX_UA_NODE")

    # 2. push kafka client cert to k8s platform
    if kafka_certs:
        _push_certs(host, kafka_certs, "NSX_UA_KAFKA_CLIENT")

    # sleep for a while since some services will be restarted
    time.sleep(30)


def _push_certs(host: str, certs_list: list, used_by: str):
    for cert in certs_list:
        body = {
            "alias": cert['id'],
            "pem_encoded": cert['pem_encoded'],
            "used_by": used_by,
            "skip_sync_services": True
        }
        assert_request_success(
            remote_node_request_with_retry(host, "POST",
                                           POST_CLOUDNATIVE_PLATFROM_CERT,
                                           body=body, max_retry=3),
            "POST: %s" % POST_CLOUDNATIVE_PLATFROM_CERT)

    # restart services
    body = {
        "used_by": used_by,
        "skip_sync_services": False
    }
    assert_request_success(
        remote_node_request_with_retry(host, "POST",
                                       POST_CLOUDNATIVE_PLATFROM_CERT,
                                       body=body, max_retry=3),
        "POST: %s" % POST_CLOUDNATIVE_PLATFROM_CERT)


def _clean_up_old_certs():
    _unregister_kafka_client_cert()
    _unregister_cloudnative_platform_certs()


def _clean_up_work_dir():
    work_dir = Path(WORK_DIR)
    if work_dir.exists():
        shutil.rmtree(work_dir)


def _mount_ca_certificates_to_cluster_api(namespace: str):
    """
    If customer has private/air-gapped harbor, with self-signed or CA signed
    certificate - this method will:
    1. read all certs (in case they have more than 1) from the pre-defined
       location "/usr/local/share/ca-certificates"
    2. create a "harbor-certs" secret in nsxi-platform namespace with all the certs
    3. create a yaml patch containing the edit for cluster-api pod to mount the
       secret and mount individual keys in the secret as a separate file
       inside "/etc/ssl/certs" location - without mounting the complete volume for
       "/etc/ssl/certs" - so pre-packaged files in that location remain as-is
    4. apply the secret and patch yaml to k8s cluster

    :param namespace: namespace in which to create/patch the resources
    :return: void
    """
    # sanity check - does the PATH_CA_CERTIFICATE_DIR exist and is non-empty
    if os.path.isdir(PATH_CA_CERTIFICATE_DIR):
        if not os.listdir(PATH_CA_CERTIFICATE_DIR):
            lg.info("%s directory is empty", PATH_CA_CERTIFICATE_DIR)
            return
    else:
        lg.info("%s directory is not present", PATH_CA_CERTIFICATE_DIR)
        return
    # directory is present and is non-empty, continue
    volume_definition_yaml = yaml.safe_load(YAML_VOLUME_DEFINITION)
    volume_definition_list = list()
    volume_mounts_yaml = yaml.safe_load(YAML_VOLUME_MOUNT)
    volume_mounts_list = list()
    harbor_secret_yaml = yaml.safe_load(YAML_SECRET_HARBOR)
    harbor_secret_yaml["data"] = dict()
    for idx, filename in enumerate(os.listdir(PATH_CA_CERTIFICATE_DIR)):
        cert = os.path.join(PATH_CA_CERTIFICATE_DIR, filename)
        if os.path.isfile(cert):
            # each cert file may contain 1 or more certs (chain)
            cert_name = f'harborcert{idx}'
            cert_name_pem = f'harborcert{idx}.pem'
            file_text = open(cert, 'rb')
            b64_cert = base64.b64encode(file_text.read())
            # create a key in the secret
            harbor_secret_yaml["data"][cert_name] = b64_cert.decode('ascii')
            # create a volume definition and append to definition list
            volume_definition = copy.deepcopy(volume_definition_yaml)
            volume_definition["name"] = cert_name
            volume_definition["secret"]["items"][0]["key"] = cert_name
            volume_definition["secret"]["items"][0]["path"] = cert_name_pem
            volume_definition_list.append(volume_definition)
            # create a volume mount and append to volume mounts list
            volume_mount = copy.deepcopy(volume_mounts_yaml)
            volume_mount["name"] = cert_name
            volume_mount["mountPath"] = f'/etc/ssl/certs/{cert_name_pem}'
            volume_mount["subPath"] = cert_name_pem
            volume_mounts_list.append(volume_mount)

    lg.info("writing harbor-certs secret yaml file")
    with open(f'/tmp/secret.yaml', 'w') as file:
        yaml.dump(harbor_secret_yaml, file)

    # apply the secret.yaml template to cluster
    lg.info("creating harbor-certs secret in k8s cluster")
    command = "apply -f /tmp/secret.yaml -n %s" % namespace
    rc, out, error = kubectl_command(command)
    if rc != 0:
        lg.error("Failed to create harbor-certs secret %s" % error)
        raise RuntimeError("Failed to create harbor-certs secret %s" % error)

    deployment_patch = yaml.safe_load(YAML_DEPLOYMENT_PATCH)
    deployment_patch["spec"]["template"]["spec"]["volumes"] = volume_definition_list
    deployment_patch["spec"]["template"]["spec"]["containers"][0]["volumeMounts"] = volume_mounts_list
    lg.info("writing cluster-api deployment patch yaml file")
    with open(f'/tmp/patch.yaml', 'w') as file:
        yaml.dump(deployment_patch, file)

    lg.info("applying patch to cluster-api deployment")
    command = "patch deployment cluster-api --patch-file /tmp/patch.yaml -n %s" % namespace
    rc, out, error = kubectl_command(command)
    if rc != 0:
        lg.error("Failed to apply patch to cluster-api deployment %s" % error)
        raise RuntimeError("Failed to apply patch to cluster-api deployment %s" % error)


def main(arguments):
    validate_k8s_compatibility()
    _mount_ca_certificates_to_cluster_api(arguments.namespace)
    fqdn = _get_ingress_fqdn(arguments.namespace)
    _register_cloudnative_platform_certs(fqdn)
    _register_kafka_client_cert()
    _register_manager_certs(fqdn)
    _wait_util_platform_ready(fqdn)
    _get_features_intelligence_config(fqdn)
    lg.info("Registration Successfully!!!")


if __name__ == "__main__":
    try:
        parser = argparse.ArgumentParser(description="Registration between "
                                                     "NSX Application "
                                                     "Platform and NSX "
                                                     "Manager")
        parser.add_argument('-u', "--unregister", help="cleanup NAPP "
                                                       "certificates on NSX "
                                                       "Manager",
                            required=False, action='store_true')
        parser.add_argument("-n", "--namespace", help="the namespace where "
                                                      "platform chart is "
                                                      "installed",
                            required=('-u' not in sys.argv and
                                      '--unregister' not in sys.argv))
        parser.add_argument("-c", "--cmnamespace", help="the namespace where "
                                                        "cert-manager is "
                                                        "installed",
                            default="cert-manager", required=False)
        parser.add_argument("-k", "--kubeconfig", help="Kubeconfig, the file "
                                                       "used to configure "
                                                       "access to Kubernetes "
                                                       "(Optional)",
                            default="~/.kube/config", required=False)
        args = parser.parse_args()
        # setup logger
        setup_logger()
        if args.unregister:
            _clean_up_old_certs()
            sys.exit(0)
        # cleanup
        _clean_up_work_dir()
        _clean_up_old_certs()
        # setup kubectl
        set_kubectl(args.kubeconfig)
        # setup cert manager namespace
        set_cert_manager_ns(args.cmnamespace)
        # setup work dir
        work_dir_path = Path(WORK_DIR)
        os.mkdir(work_dir_path)
        # main func
        main(args)
    except Exception as e:
        lg.exception("Exit unexpectedly")
        sys.exit(-1)
    finally:
        _clean_up_work_dir()
